#include <cstddef>
using std::size_t;

#include <cstdlib>
using std::exit;
using std::atof;

#include <cassert>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <iostream>
using std::cerr;
using std::endl;

#include <random>
using std::mt19937;
using std::uniform_real_distribution;

#include <chrono>
using std::chrono::high_resolution_clock;

#include <functional>
using std::bind;

static mt19937 rng; // mersenne twistor random number generator
static auto rnd = bind(uniform_real_distribution<double>(0,1),rng); // random number in [0,1)

const size_t halfL = 200;
const size_t L = 2*halfL;
const double d = 1.0/L;
enum cell_t { UNASSIGNED = 2, ON = 1, OFF = 0 };
char grid[L][L];
const int delay = 2; // milliseconds

void display()
{
	glClear(GL_COLOR_BUFFER_BIT);
	for (size_t i = 0; i < L; ++i)
		for (size_t j = 0; j < L; ++j)
			if (grid[i][j] != UNASSIGNED)
			{
				if (grid[i][j] == ON)
					glColor3f(0.0, 0.0, 0.0); 
				else
					glColor3f(0.4, 0.75, 0.8); 
				const double x = 2.0*(i+0.5)/L - 1.0;
				const double y = 2.0*(j+0.5)/L - 1.0;	
				glBegin(GL_POLYGON);
					glVertex2f(x-d,y-d);
					glVertex2f(x-d,y+d);
					glVertex2f(x+d,y+d);
					glVertex2f(x+d,y-d);
				glEnd();
				glFlush();
			}
	glutSwapBuffers();
}

void initialize_grid(void)
{
	for (size_t i = 0; i < L; ++i)
		for (size_t j = 0; j < L; ++j)
			grid[i][j] = UNASSIGNED;
	grid[halfL][halfL] = ON;
}

double prob;
void sweep_grid(void)
{
	for (size_t i = 0; i < L; ++i)
		for (size_t j = 0; j < L; ++j)
			if ( grid[i][j] == UNASSIGNED and (grid[(i+1)%L][j] == ON or grid[(L+i-1)%L][j] == ON or
											   grid[i][(j+1)%L] == ON or grid[i][(L+j-1)%L] == ON) )
				grid[i][j] = (rnd() < prob ? ON : OFF);
}

void animate(int)
{
	sweep_grid();
	glutPostRedisplay();
	glutTimerFunc(delay,animate,0);
}

int main(int argc, char **argv)
{
	if (argc != 2)
	{
		cerr << "Usage: perc_cluster (0 < prob < 1)" << endl;
		exit(1);
	}
	prob = atof(argv[1]);	
	
   rng.seed(mt19937::result_type(high_resolution_clock::now().time_since_epoch().count()));
   
	glutInit(&argc,argv);
	glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowSize (640, 640);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Grow a percolating cluster");
	glutDisplayFunc(display);
	glutTimerFunc(delay,animate,0);
	initialize_grid();
	glClearColor(1.0, 1.0, 1.0, 1.0);
	//glutPostRedisplay();
	glutMainLoop();
}
